@@ -99,7 +99,10 @@ struct dnsrbl default_dnsrbl[] = {
 	{ "bl.blocklist.de",		"127.0.0.1/8" },
 	{ "srnblack.surgate.net",	"127.0.0.1/8" },
 	{ "all.s5h.net",		"127.0.0.1/8" },
-	{ "rbl.megarbl.net",		"127.0.0.1/8" },
+	/*
+	 * Broken as of 2025-11-15
+	 * { "rbl.megarbl.net",		"127.0.0.1/8" },
+	 */
 	{ "rbl.realtimeblacklist.com",	"127.0.0.1/8" },
 	{ "b.barracudacentral.org",	"127.0.0.1/8" },
 	{ "dnsbl.spfbl.net",		"127.0.0.1/8" },
@@ -119,7 +122,9 @@ SLIST_HEAD(addrlist_head, addrlist);
 struct rev_thread_args {
 	struct addrlist_head	*rta_alp;
 	pthread_mutex_t		*rta_mtx;
-	char			*rta_addr;
+	char			*rta_query;
+	unsigned char		*rta_addr;
+	int			rta_family;
 	pthread_cond_t		*rta_cond;
 	int			*rta_count;
 };
@@ -147,7 +152,7 @@ void usage(char *);
 void parse_options(int, char **);
 void dump_dnsrbl(void);
 struct dnsrbl *load_dnsrbl(char *);
-void rev_addr(struct addrlist_head *, pthread_mutex_t *, char *);
+void rev_addr(struct addrlist_head *, pthread_mutex_t *, char *, unsigned char *, int);
 void rev_dns(struct addrlist_head *, pthread_mutex_t *, char *);
 void *rev_addr_thread(void *);
 void *rev_dns_thread(void *);
@@ -245,13 +250,46 @@ parse_options(argc, argv)
 	return;
 }
 
+void
+print_rev4(s, a)
+	char *s;
+	unsigned char *a;
+{
+	sprintf(s, "%d.%d.%d.%d",  
+		a[3], a[2], a[1], a[0]);
+}
+
+void
+print_rev6(s, a)
+	char *s;
+	unsigned char *a;
+{
+	sprintf(s, "%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x.%x",  
+	        a[15] & 0xf, a[15] >> 4,
+	        a[14] & 0xf, a[14] >> 4,
+	        a[13] & 0xf, a[13] >> 4,
+	        a[12] & 0xf, a[12] >> 4,
+	        a[11] & 0xf, a[11] >> 4,
+	        a[10] & 0xf, a[10] >> 4,
+	        a[9] & 0xf, a[9] >> 4,
+	        a[8] & 0xf, a[8] >> 4,
+	        a[7] & 0xf, a[7] >> 4,
+	        a[6] & 0xf, a[6] >> 4,
+	        a[5] & 0xf, a[5] >> 4,
+	        a[4] & 0xf, a[4] >> 4,
+	        a[3] & 0xf, a[3] >> 4,
+	        a[2] & 0xf, a[2] >> 4,
+	        a[1] & 0xf, a[1] >> 4,
+	        a[0] & 0xf, a[0] >> 4);
+}
+
 void *
 rev_addr_thread(void *args)
 {
 	struct rev_thread_args *rta = args;
 	int *count = rta->rta_count;
 
-	rev_addr(rta->rta_alp, rta->rta_mtx, rta->rta_addr);
+	rev_addr(rta->rta_alp, rta->rta_mtx, rta->rta_query, rta->rta_addr, rta->rta_family);
 
 	if (pthread_mutex_lock(rta->rta_mtx) != 0)
 		panic("pthread_mutex_lock failed");
@@ -264,42 +302,35 @@ rev_addr_thread(void *args)
 	if (pthread_cond_signal(rta->rta_cond) != 0)
 		panic("pthread_cond_signal failed");
 
+	free(rta->rta_query);
 	free(rta->rta_addr);
 	free(rta);
 	return NULL;
 }
 
 void
-rev_addr(alp, mtx, addr)
+rev_addr(alp, mtx, query, addr, family)
 	struct addrlist_head *alp;
 	pthread_mutex_t *mtx;
-	char *addr;
+	char *query;
+	unsigned char *addr;
+	int family;
 {
-	int s, d, l, len;
 	struct addrlist *al;
-	char *dst;
 
 
 	if ((al = malloc(sizeof(*al))) == NULL)
 		panic("malloc failed");
 
-	if ((al->al_query = strdup(addr)) == NULL)
+	if ((al->al_query = strdup(query)) == NULL)
 		panic("strdup failed");
-	dst = al->al_addr;
 
-	len = strlen(addr) - 1;
-	l = len;
-	d = 0;
-	for (s = len; s >= 0; s--) {
-		if (addr[s] == '.') {
-			memcpy(dst + d, addr + s + 1, l - s);
-			d += l - s;
-			dst[d++] = '.';
-			l = s - 1;
-		}
-	}
-
-	memcpy(dst + d, addr + s + 1, l - s);
+	if (family == AF_INET)
+		print_rev4(al->al_addr, addr);
+	else if (family == AF_INET6)
+		print_rev6(al->al_addr, addr);
+	else
+		panic("rta_family");
 
 	if (pthread_mutex_lock(mtx) != 0)
 		panic("pthread_mutex_lock failed");
@@ -396,23 +427,33 @@ load_addr(alp, mtx, addrs)
 
 		rta->rta_alp = alp;
 		rta->rta_mtx = mtx;
-		rta->rta_addr = strdup(a); /* thread is to free it */
+		rta->rta_query = strdup(a); /* thread is to free it */
 		rta->rta_cond = &wait_cond;
 		rta->rta_count = &count;
 
-		if (rta->rta_addr == NULL)
+		if (rta->rta_query == NULL)
 			panic("strdup failed");
+		if ((rta->rta_addr = malloc(16)) == NULL)
+			panic("malloc failed");
 
 		if (verbose)
-			printf("lookup %s\n", rta->rta_addr);
+			printf("lookup %s\n", rta->rta_query);
 
-		if (inet_addr(rta->rta_addr) == INADDR_NONE)
-			res = pthread_create(&thread, NULL,
-					     *rev_dns_thread, rta);
-		else
+		if (inet_pton(AF_INET, rta->rta_query, rta->rta_addr) == 1) {
+			rta->rta_family = AF_INET;
 			res = pthread_create(&thread, NULL,
 					     *rev_addr_thread, rta);
-
+		} else if (inet_pton(AF_INET6, rta->rta_query, rta->rta_addr) == 1) {
+			rta->rta_family = AF_INET6;
+			res = pthread_create(&thread, NULL,
+					     *rev_addr_thread, rta);
+		} else {
+			rta->rta_family = AF_UNSPEC;
+			free(rta->rta_addr);
+			res = pthread_create(&thread, NULL,
+					     *rev_dns_thread, rta);
+		}
+			
 		if (res != 0)
 			panic("pthread_create failed");
 
@@ -450,7 +491,7 @@ rev_dns_thread(void *args)
 	struct rev_thread_args *rta = args;
 	int *count = rta->rta_count;
 
-	rev_dns(rta->rta_alp, rta->rta_mtx, rta->rta_addr);
+	rev_dns(rta->rta_alp, rta->rta_mtx, rta->rta_query);
 
 	if (pthread_mutex_lock(rta->rta_mtx) != 0)
 		panic("pthread_mutex_lock failed");
@@ -463,7 +504,7 @@ rev_dns_thread(void *args)
 	if (pthread_cond_signal(rta->rta_cond) != 0)
 		panic("pthread_cond_signal failed");
 
-	free(rta->rta_addr);
+	free(rta->rta_query);
 	free(rta);
 	return NULL;
 }
@@ -506,9 +547,7 @@ rev_dns(alp, mtx, name)
 
 			if ((al->al_query = strdup(name)) == NULL)
 				panic("strdup failed");
-			sprintf(al->al_addr, "%d.%d.%d.%d",  
-				rr.rdata[3], rr.rdata[2], 
-				rr.rdata[1], rr.rdata[0]);
+			print_rev4(al->al_addr, rr.rdata);
 			
 			if (pthread_mutex_lock(mtx) != 0)
 				panic("pthread_mutex_lock failed");
@@ -530,7 +569,7 @@ rev_dns(alp, mtx, name)
 	 */
 	if (anslen6 != -1) {
 		if (ns_initparse(ans6, anslen6, &handle) != 0)
-			panic("ns_initparse IPv4 failed");
+			panic("ns_initparse IPv6 failed");
 
 		for (i = 0; i < ns_msg_count(handle, ns_s_an); i++) {
 			if ((ns_parserr(&handle, ns_s_an, i, &rr)) != 0) 
@@ -544,17 +583,8 @@ rev_dns(alp, mtx, name)
 
 			if ((al->al_query = strdup(name)) == NULL)
 				panic("strdup failed");
-			sprintf(al->al_addr, 
-			    "%d.%d.%d.%d.%d.%d.%d.%d.%d.%d.%d.%d.%d.%d.%d.%d",
-			     rr.rdata[15], rr.rdata[14],
-			     rr.rdata[13], rr.rdata[12],
-			     rr.rdata[11], rr.rdata[10],
-			     rr.rdata[9], rr.rdata[8],
-			     rr.rdata[7], rr.rdata[6],
-			     rr.rdata[5], rr.rdata[4],
-			     rr.rdata[3], rr.rdata[3],
-			     rr.rdata[1], rr.rdata[0]);
-				
+			print_rev6(al->al_addr, rr.rdata);
+
 			if (pthread_mutex_lock(mtx) != 0)
 				panic("pthread_mutex_lock failed");
 
